@@ -1,8 +1,9 @@
 import collections
-import hashlib
+import functools
 import logging
 import os
 import pathlib
+import secrets
 import shutil
 
 import event_model
@@ -10,6 +11,7 @@ import databroker.core
 from tqdm import tqdm
 import yaml
 
+from ._utils import root_hash
 from ._version import get_versions
 
 __all__ = (
@@ -37,9 +39,10 @@ def export_uids(
     *,
     strict=False,
     external=None,
-    dry_run=False,
+    no_documents=False,
     handler_registry=None,
     serializer_class=None,
+    salt=None,
 ):
     """
     Export Runs from a Catalog, given a list of RunStart unique IDs.
@@ -59,7 +62,7 @@ def export_uids(
         If None, return the paths to external files.
         If 'fill', fill the external data into the Documents.
         If 'ignore', do not locate external files.
-    dry_run: Bool, optional
+    no_documents: Bool, optional
         If True, do not write any files. False by default.
     handler_registry: Union[Dict, None]
         If None, automatic handler discovery is used.
@@ -69,15 +72,37 @@ def export_uids(
         currently ``suitcase.msgpack.Serializer``, but this may change in the
         future. If you want ``suitcase.msgpack.Serializer`` specifically, pass
         it in explicitly.
+    salt: Union[bytes, None]
+        We want to make hashes is unique to:
+
+        - a root
+        - a given batch of exported runs (i.e., a given call to this function)
+
+        so that we can use it as a key in root_map which is guaranteed not to
+        collide with keys from other batches. Thus, we create a "salt" unless
+        one is specified here. This does not need to be cryptographically
+        secure, just unique.
 
     Returns
     -------
-    files: Dict[Str, Set[Str]]
-        Maps each "root" to a set of absolute file paths.
+    artifacts, files, failures
+
+    Notes
+    -----
+    * ``artifacts`` maps a human-readable string (typically just ``'all'`` in
+      this case) to a list of buffers or filepaths where the documents were
+      serialized.
+    * ``files`` is the set of filepaths of all external files referenced by
+      Resource documents, keyed on ``(root_in_document, root, unique_id)``.
+    * ``failures`` is a list of uids of runs that raised Exceptions. (The
+      relevant tracebacks are logged.)
     """
     accumulated_files = collections.defaultdict(set)
-    accumulated_artifacts = collections.defaultdict(list)
+    accumulated_artifacts = collections.defaultdict(set)
     failures = []
+    if salt is None:
+        salt = secrets.token_hex(32).encode()
+    root_hash_func = functools.partial(root_hash, salt)
     with tqdm(total=len(uids), position=1, desc="Writing Documents") as progress:
         for uid in uids:
             try:
@@ -85,8 +110,9 @@ def export_uids(
                 artifacts, files = export_run(
                     run,
                     directory,
+                    root_hash_func,
                     external=external,
-                    dry_run=dry_run,
+                    no_documents=no_documents,
                     handler_registry=handler_registry,
                     root_map=source_catalog.root_map,
                     serializer_class=serializer_class,
@@ -94,7 +120,7 @@ def export_uids(
                 for root, set_ in files.items():
                     accumulated_files[root].update(set_)
                 for name, list_ in artifacts.items():
-                    accumulated_artifacts[name].extend(list_)
+                    accumulated_artifacts[name].update(list_)
 
             except Exception:
                 logger.exception("Error while exporting Run %r", uid)
@@ -114,9 +140,11 @@ def export_catalog(
     *,
     strict=False,
     external=None,
-    dry_run=False,
+    no_documents=False,
     handler_registry=None,
     serializer_class=None,
+    salt=None,
+    limit=None,
 ):
     """
     Export all the Runs from a Catalog.
@@ -134,8 +162,8 @@ def export_catalog(
         If None, return the paths to external files.
         If 'fill', fill the external data into the Documents.
         If 'ignore', do not locate external files.
-    dry_run: Bool, optional
-        If True, do not write any files. False by default.
+    no_documents: Bool, optional
+        If True, do not serialize documents. False by default.
     handler_registry: Union[Dict, None]
         If None, automatic handler discovery is used.
     serializer_class: Serializer
@@ -144,25 +172,57 @@ def export_catalog(
         currently ``suitcase.msgpack.Serializer``, but this may change in the
         future. If you want ``suitcase.msgpack.Serializer`` specifically, pass
         it in explicitly.
+    salt: Union[bytes, None]
+        We want to make hashes is unique to:
+
+        - a root
+        - a given batch of exported runs (i.e., a given call to this function)
+
+        so that we can use it as a key in root_map which is guaranteed not to
+        collide with keys from other batches. Thus, we create a "salt" unless
+        one is specified here. This does not need to be cryptographically
+        secure, just unique.
+    limit: Union[Integer, None]
+        Stop after exporting some number of Runs. Useful for testing a subset
+        before doing a lengthy export.
 
     Returns
     -------
-    files: Dict[Str, Set[Str]]
-        Maps each "root" to a set of absolute file paths.
+    artifacts, files, failures
+
+    Notes
+    -----
+    * ``artifacts`` maps a human-readable string (typically just ``'all'`` in
+      this case) to a list of buffers or filepaths where the documents were
+      serialized.
+    * ``files`` is the set of filepaths of all external files referenced by
+      Resource documents, keyed on ``(root_in_document, root, unique_id)``.
+    * ``failures`` is a list of uids of runs that raised Exceptions. (The
+      relevant tracebacks are logged.)
     """
+    if limit is not None:
+        if limit < 1:
+            raise ValueError("limit must be None or a number 1 or greater")
+        limit = int(limit)
     accumulated_files = collections.defaultdict(set)
-    accumulated_artifacts = collections.defaultdict(list)
+    accumulated_artifacts = collections.defaultdict(set)
     failures = []
+    if salt is None:
+        salt = secrets.token_hex(32).encode()
+    root_hash_func = functools.partial(root_hash, salt)
     with tqdm(
-        total=len(source_catalog), position=1, desc="Writing Documents"
+        total=limit or len(source_catalog), position=1, desc="Writing Documents"
     ) as progress:
-        for uid, run in source_catalog.items():
+        for i, (uid, run) in enumerate(source_catalog.items()):
+            if i == limit:
+                break
             try:
                 artifacts, files = export_run(
                     run,
                     directory,
+                    root_hash_func,
                     external=external,
-                    dry_run=dry_run,
+                    no_documents=no_documents,
                     handler_registry=handler_registry,
                     root_map=source_catalog.root_map,
                     serializer_class=serializer_class,
@@ -170,7 +230,7 @@ def export_catalog(
                 for root, set_ in files.items():
                     accumulated_files[root].update(set_)
                 for name, list_ in artifacts.items():
-                    accumulated_artifacts[name].extend(list_)
+                    accumulated_artifacts[name].update(list_)
             except Exception:
                 logger.exception("Error while exporting Run %r", uid)
                 if strict:
@@ -186,9 +246,10 @@ def export_catalog(
 def export_run(
     run,
     directory,
+    root_hash_func,
     *,
     external=None,
-    dry_run=False,
+    no_documents=False,
     handler_registry=None,
     root_map=None,
     serializer_class=None,
@@ -206,8 +267,8 @@ def export_run(
         If None, return the paths to external files.
         If 'fill', fill the external data into the Documents.
         If 'ignore', do not locate external files.
-    dry_run: Bool, optional
-        If True, do not write any files. False by default.
+    no_documents: Bool, optional
+        If True, do not serialize documents. False by default.
     handler_registry: Union[Dict, None]
         If None, automatic handler discovery is used.
     serializer_class: Serializer, optional
@@ -219,61 +280,89 @@ def export_run(
 
     Returns
     -------
-    files: Dict[Str, Set[Str]]
-        Maps each "root" to a set of absolute file paths.
+    artifacts, files
+
+    Notes
+    -----
+    * ``artifacts`` maps a human-readable string (typically just ``'all'`` in
+      this case) to a list of buffers or filepaths where the documents were
+      serialized.
+    * ``files`` is the set of filepaths of all external files referenced by
+      Resource documents, keyed on ``(root_in_document, root, unique_id)``.
     """
+    EXTERNAL_RELATED_DOCS = ("resource", "datum", "datum_page")
     if serializer_class is None:
         import suitcase.msgpack
 
         serializer_class = suitcase.msgpack.Serializer
     root_map = root_map or {}
-    resources = []
     files = collections.defaultdict(set)
     if handler_registry is None:
         handler_registry = databroker.core.discover_handlers()
     with event_model.Filler(
         handler_registry, inplace=False, root_map=root_map
     ) as filler:
-        with serializer_class(directory) as serializer:
+        with serializer_class(
+            directory, file_prefix="documents/{start[uid]}"
+        ) as serializer:
             with tqdm(position=0) as progress:
                 for name, doc in run.canonical(fill="no"):
-                    if name == "resource":
-                        resources.append(doc)
                     if external == "fill":
                         name, doc = filler(name, doc)
-                    if not dry_run:
+                        # Omit Resource and Datum[Page] because the data was
+                        # filled in place.
+                        if name in EXTERNAL_RELATED_DOCS:
+                            progress.update()
+                            continue
+                    elif name == "resource":
+                        root = root_map.get(doc["root"], doc["root"])
+                        unique_id = root_hash_func(doc["root"])
+                        if external is None:
+                            if no_documents:
+                                root_in_document = doc["root"]
+                            else:
+                                root_in_document = root
+                            # - root_in_document is the 'root' actually in the
+                            # resource_document
+                            # - root may be different depending on the
+                            # source_catalog configuration, which can map the
+                            # recorded 'root' in the document to some other
+                            # location. This is where we should go looking for
+                            # the data if we plan to copy it.
+                            # - unique_id is unique to this (root, salt)
+                            # combination and used to place the data in a
+                            # unique location.
+                            key = (root_in_document, root, unique_id)
+                            files[key].update(run.get_file_list(doc))
+                        if not no_documents:
+                            # Replace root with a unique ID before serialization.
+                            # We are overriding the local variable name doc here
+                            # (yuck!) so that serializer(name, doc) below works on
+                            # all document types.
+                            doc = doc.copy()
+                            doc["root"] = unique_id
+                    if not no_documents:
                         serializer(name, doc)
                     progress.update()
-        if external is None:
-            for resource in resources:
-                root = root_map.get(resource["root"], resource["root"])
-                files[root].update(run.get_file_list(resource))
     return serializer.artifacts, dict(files)
 
 
-def _root_hash(root):
-    # This is just a unique ID to give the manifest file for each
-    # root a unique name. It is not a cryptographic hash.
-    return hashlib.md5(root.encode()).hexdigest()
-
-
-def write_external_files_manifest(manager, root, files):
+def write_external_files_manifest(manager, unique_id, files):
     """
     Write a manifest of external files.
 
     Parameters
     ----------
     manager: suitcase Manager object
-    root: Str
+    unique_id: Str
     files: Iterable[Union[Str, Path]]
     """
-    root_hash = _root_hash(root)
-    name = f"external_files_manifest_{root_hash}.txt"
+    name = f"external_files_manifest_{unique_id}.txt"
     with manager.open("manifest", name, "xt") as file:
         file.write("\n".join(sorted((str(f) for f in set(files)))))
 
 
-def copy_external_files(target_directory, root, files, strict=False):
+def copy_external_files(target_directory, root, unique_id, files, strict=False):
     """
     Make a filesystem copy of the external files.
 
@@ -295,14 +384,19 @@ def copy_external_files(target_directory, root, files, strict=False):
     Returns
     -------
     new_root, new_files, failures
+
+    Notes
+    -----
+    * ``new_root`` is a Path to the new root directory
+    * ``new_files`` is the list of filepaths to the files that were created.
+    * ``failures`` is a list of uids of runs that raised Exceptions. (The
+      relevant tracebacks are logged.)
     """
-    root_hash = _root_hash(root)
-    dest = str(pathlib.Path(target_directory, root_hash))
     new_files = []
     failures = []
     for filename in tqdm(files, total=len(files), desc="Copying external files"):
         relative_path = pathlib.Path(filename).relative_to(root)
-        new_root = pathlib.Path(target_directory, root_hash)
+        new_root = pathlib.Path(target_directory, unique_id)
         dest = new_root / relative_path
         try:
             os.makedirs(dest.parent, exist_ok=True)
